@@ -179,37 +179,41 @@ use File::Glob qw(:case);
     },
 
     'base' => {
-        'DISTRIBUTION' => 'RGARCIA/base-2.18.tar.gz',
+        'DISTRIBUTION' => 'RJBS/base-2.23.tar.gz',
         'FILES'        => q[dist/base],
     },
 
     'bignum' => {
-        'DISTRIBUTION' => 'PJACKLAM/bignum-0.37.tar.gz',
-        'FILES'        => q[dist/bignum],
+        'DISTRIBUTION' => 'PJACKLAM/bignum-0.41.tar.gz',
+        'FILES'        => q[cpan/bignum],
         'EXCLUDED'     => [
             qr{^inc/Module/},
-            qw( t/pod.t
-                t/pod_cov.t
+            qw( t/00sig.t
+                t/01load.t
+                t/02pod.t
+                t/03podcov.t
                 ),
         ],
     },
 
     'Carp' => {
-        'DISTRIBUTION' => 'RJBS/Carp-1.36.tar.gz',
+        'DISTRIBUTION' => 'RJBS/Carp-1.38.tar.gz',
         'FILES'        => q[dist/Carp],
     },
 
     'Compress::Raw::Bzip2' => {
-        'DISTRIBUTION' => 'PMQS/Compress-Raw-Bzip2-2.068.tar.gz',
+        'DISTRIBUTION' => 'PMQS/Compress-Raw-Bzip2-2.069.tar.gz',
         'FILES'        => q[cpan/Compress-Raw-Bzip2],
         'EXCLUDED'     => [
             qr{^t/Test/},
+            'bzip2-src/bzip2-const.patch',
             'bzip2-src/bzip2-cpp.patch',
+            'bzip2-src/bzip2-unsigned.patch',
         ],
     },
 
     'Compress::Raw::Zlib' => {
-        'DISTRIBUTION' => 'PMQS/Compress-Raw-Zlib-2.068.tar.gz',
+        'DISTRIBUTION' => 'PMQS/Compress-Raw-Zlib-2.069.tar.gz',
 
         'FILES'    => q[cpan/Compress-Raw-Zlib],
         'EXCLUDED' => [
@@ -222,7 +226,7 @@ use File::Glob qw(:case);
     },
 
     'Config::Perl::V' => {
-        'DISTRIBUTION' => 'HMBRAND/Config-Perl-V-0.24.tgz',
+        'DISTRIBUTION' => 'HMBRAND/Config-Perl-V-0.25.tgz',
         'FILES'        => q[cpan/Config-Perl-V],
         'EXCLUDED'     => [qw(
 		examples/show-v.pl
@@ -299,7 +303,7 @@ use File::Glob qw(:case);
     },
 
     'CPAN::Meta::Requirements' => {
-        'DISTRIBUTION' => 'DAGOLDEN/CPAN-Meta-Requirements-2.133.tar.gz',
+        'DISTRIBUTION' => 'DAGOLDEN/CPAN-Meta-Requirements-2.140.tar.gz',
         'FILES'        => q[cpan/CPAN-Meta-Requirements],
         'EXCLUDED'     => [
             qw(t/00-report-prereqs.t),
@@ -310,7 +314,7 @@ use File::Glob qw(:case);
     },
 
     'CPAN::Meta::YAML' => {
-        'DISTRIBUTION' => 'DAGOLDEN/CPAN-Meta-YAML-0.017-TRIAL.tar.gz',
+        'DISTRIBUTION' => 'DAGOLDEN/CPAN-Meta-YAML-0.018.tar.gz',
         'FILES'        => q[cpan/CPAN-Meta-YAML],
         'EXCLUDED'     => [
             't/00-report-prereqs.t',
@@ -337,7 +341,7 @@ use File::Glob qw(:case);
     },
 
     'Devel::PPPort' => {
-        'DISTRIBUTION' => 'WOLFSAGE/Devel-PPPort-3.31.tar.gz',
+        'DISTRIBUTION' => 'WOLFSAGE/Devel-PPPort-3.32.tar.gz',
         # RJBS has asked MHX to have UPSTREAM be 'blead'
         # (i.e. move this from cpan/ to dist/)
         'FILES'        => q[cpan/Devel-PPPort],
@@ -382,13 +386,19 @@ use File::Glob qw(:case);
     },
 
     'Encode' => {
-        'DISTRIBUTION' => 'DANKOGAI/Encode-2.76.tar.gz',
+        'DISTRIBUTION' => 'DANKOGAI/Encode-2.78.tar.gz',
         'FILES'        => q[cpan/Encode],
+        CUSTOMIZED     => [
+            qw( Encode.xs
+                Unicode/Unicode.xs
+                encoding.pm
+                ),
+        ],
     },
 
     'encoding::warnings' => {
         'DISTRIBUTION' => 'AUDREYT/encoding-warnings-0.11.tar.gz',
-        'FILES'        => q[cpan/encoding-warnings],
+        'FILES'        => q[dist/encoding-warnings],
         'EXCLUDED'     => [
             qr{^inc/Module/},
             qw(t/0-signature.t),
@@ -402,12 +412,9 @@ use File::Glob qw(:case);
     },
 
     'experimental' => {
-        'DISTRIBUTION' => 'LEONT/experimental-0.013.tar.gz',
+        'DISTRIBUTION' => 'LEONT/experimental-0.016.tar.gz',
         'FILES'        => q[cpan/experimental],
-        'EXCLUDED'     => [
-          qr{^t/release-.*\.t},
-          't/00-compile.t',
-        ],
+        'EXCLUDED'     => [qr{^xt/}],
     },
 
     'Exporter' => {
@@ -421,7 +428,7 @@ use File::Glob qw(:case);
     },
 
     'ExtUtils::CBuilder' => {
-        'DISTRIBUTION' => 'AMBS/ExtUtils-CBuilder-0.280223.tar.gz',
+        'DISTRIBUTION' => 'AMBS/ExtUtils-CBuilder-0.280224.tar.gz',
         'FILES'        => q[dist/ExtUtils-CBuilder],
         'EXCLUDED'     => [
             qw(README.mkdn),
@@ -429,18 +436,9 @@ use File::Glob qw(:case);
         ],
     },
 
-    'ExtUtils::Command' => {
-        'DISTRIBUTION' => 'BINGOS/ExtUtils-Command-1.20.tar.gz',
-        'FILES'        => q[cpan/ExtUtils-Command],
-        'EXCLUDED'     => [qr{^xt/}],
-    },
-
     'ExtUtils::Constant' => {
 
-        # Nick has confirmed that while we have diverged from CPAN,
-        # this package isn't primarily maintained in core
-        # Another release will happen "Sometime"
-        'DISTRIBUTION' => '',    #'NWCLARK/ExtUtils-Constant-0.16.tar.gz',
+        'DISTRIBUTION' => 'NWCLARK/ExtUtils-Constant-0.23.tar.gz',
         'FILES'    => q[cpan/ExtUtils-Constant],
         'EXCLUDED' => [
             qw( lib/ExtUtils/Constant/Aaargh56Hash.pm
@@ -448,6 +446,8 @@ use File::Glob qw(:case);
                 examples/perl_regcomp_posix_keyword.pl
                 ),
         ],
+        # cc37ebcee3 to fix VMS failure
+        'CUSTOMIZED'   => [ qw(t/Constant.t) ],
     },
 
     'ExtUtils::Install' => {
@@ -465,7 +465,7 @@ use File::Glob qw(:case);
     },
 
     'ExtUtils::MakeMaker' => {
-        'DISTRIBUTION' => 'BINGOS/ExtUtils-MakeMaker-7.04.tar.gz',
+        'DISTRIBUTION' => 'BINGOS/ExtUtils-MakeMaker-7.10.tar.gz',
         'FILES'        => q[cpan/ExtUtils-MakeMaker],
         'EXCLUDED'     => [
             qr{^t/lib/Test/},
@@ -475,42 +475,16 @@ use File::Glob qw(:case);
             qr{^\.perlcriticrc},
             'PATCHING',
             'README.packaging',
+            'lib/ExtUtils/MakeMaker/version/vpp.pm',
+        ],
+        # Upstreamed as https://github.com/Perl-Toolchain-Gang/ExtUtils-MakeMaker/commit/ede9ea4a
+        'CUSTOMIZED'   => [
+            qq[lib/ExtUtils/MakeMaker.pm],
+            qq[t/prereq.t],
+            qq[t/vstrings.t],
+        # Upstreamed as https://github.com/Perl-Toolchain-Gang/ExtUtils-MakeMaker/commit/dd1e236ab
+            qq[lib/ExtUtils/MM_VMS.pm],
         ],
-        # Applied upstream remove customisation when updating EUMM
-        'CUSTOMIZED'   => [ qw[ t/pm_to_blib.t
-                                t/basic.t
-                                lib/ExtUtils/Command/MM.pm
-                                lib/ExtUtils/Liblist.pm
-                                lib/ExtUtils/Liblist/Kid.pm
-                                lib/ExtUtils/MM.pm
-                                lib/ExtUtils/MM_AIX.pm
-                                lib/ExtUtils/MM_Any.pm
-                                lib/ExtUtils/MM_BeOS.pm
-                                lib/ExtUtils/MM_Cygwin.pm
-                                lib/ExtUtils/MM_DOS.pm
-                                lib/ExtUtils/MM_Darwin.pm
-                                lib/ExtUtils/MM_MacOS.pm
-                                lib/ExtUtils/MM_NW5.pm
-                                lib/ExtUtils/MM_OS2.pm
-                                lib/ExtUtils/MM_QNX.pm
-                                lib/ExtUtils/MM_UWIN.pm
-                                lib/ExtUtils/MM_Unix.pm
-                                lib/ExtUtils/MM_VMS.pm
-                                lib/ExtUtils/MM_VOS.pm
-                                lib/ExtUtils/MM_Win32.pm
-                                lib/ExtUtils/MM_Win95.pm
-                                lib/ExtUtils/MY.pm
-                                lib/ExtUtils/MakeMaker.pm
-                                lib/ExtUtils/MakeMaker/Config.pm
-                                lib/ExtUtils/MakeMaker/FAQ.pod
-                                lib/ExtUtils/MakeMaker/Locale.pm
-                                lib/ExtUtils/MakeMaker/Tutorial.pod
-                                lib/ExtUtils/MakeMaker/version.pm
-                                lib/ExtUtils/MakeMaker/version/regex.pm
-                                lib/ExtUtils/MakeMaker/version/vpp.pm
-                                lib/ExtUtils/Mkbootstrap.pm
-                                lib/ExtUtils/Mksymlists.pm
-                                lib/ExtUtils/testlib.pm ] ],
     },
 
     'ExtUtils::Manifest' => {
@@ -523,7 +497,7 @@ use File::Glob qw(:case);
     },
 
     'ExtUtils::ParseXS' => {
-        'DISTRIBUTION' => 'SMUELLER/ExtUtils-ParseXS-3.24.tar.gz',
+        'DISTRIBUTION' => 'SMUELLER/ExtUtils-ParseXS-3.30.tar.gz',
         'FILES'        => q[dist/ExtUtils-ParseXS],
     },
 
@@ -533,12 +507,11 @@ use File::Glob qw(:case);
     },
 
     'File::Path' => {
-        'DISTRIBUTION' => 'RICHE/File-Path-2.11.tar.gz',
+        'DISTRIBUTION' => 'RICHE/File-Path-2.12.tar.gz',
         'FILES'        => q[cpan/File-Path],
         'EXCLUDED'     => [
-            qw( eg/setup-extra-tests
-                t/Path-Class.t
-                )
+            qw(t/Path-Class.t),
+            qr{^xt/},
         ],
     },
 
@@ -599,7 +572,7 @@ use File::Glob qw(:case);
     },
 
     'Getopt::Long' => {
-        'DISTRIBUTION' => 'JV/Getopt-Long-2.47.tar.gz',
+        'DISTRIBUTION' => 'JV/Getopt-Long-2.48.tar.gz',
         'FILES'        => q[cpan/Getopt-Long],
         'EXCLUDED'     => [
             qr{^examples/},
@@ -635,7 +608,7 @@ use File::Glob qw(:case);
     },
 
     'if' => {
-        'DISTRIBUTION' => 'ILYAZ/modules/if-0.0601.tar.gz',
+        'DISTRIBUTION' => 'RJBS/if-0.0606.tar.gz',
         'FILES'        => q[dist/if],
     },
 
@@ -646,7 +619,7 @@ use File::Glob qw(:case);
     },
 
     'IO-Compress' => {
-        'DISTRIBUTION' => 'PMQS/IO-Compress-2.068.tar.gz',
+        'DISTRIBUTION' => 'PMQS/IO-Compress-2.069.tar.gz',
         'FILES'        => q[cpan/IO-Compress],
         'EXCLUDED'     => [
             qr{^examples/},
@@ -716,7 +689,7 @@ use File::Glob qw(:case);
     },
 
     'Locale-Codes' => {
-        'DISTRIBUTION' => 'SBECK/Locale-Codes-3.35.tar.gz',
+        'DISTRIBUTION' => 'SBECK/Locale-Codes-3.37.tar.gz',
         'FILES'        => q[cpan/Locale-Codes],
         'EXCLUDED'     => [
             qw( README.first
@@ -748,8 +721,8 @@ use File::Glob qw(:case);
     },
 
     'Math::BigInt' => {
-        'DISTRIBUTION' => 'PJACKLAM/Math-BigInt-1.9997.tar.gz',
-        'FILES'        => q[dist/Math-BigInt],
+        'DISTRIBUTION' => 'PJACKLAM/Math-BigInt-1.999710.tar.gz',
+        'FILES'        => q[cpan/Math-BigInt],
         'EXCLUDED'     => [
             qr{^inc/},
             qr{^examples/},
@@ -762,8 +735,8 @@ use File::Glob qw(:case);
     },
 
     'Math::BigInt::FastCalc' => {
-        'DISTRIBUTION' => 'PJACKLAM/Math-BigInt-FastCalc-0.31.tar.gz',
-        'FILES'        => q[dist/Math-BigInt-FastCalc],
+        'DISTRIBUTION' => 'PJACKLAM/Math-BigInt-FastCalc-0.38.tar.gz',
+        'FILES'        => q[cpan/Math-BigInt-FastCalc],
         'EXCLUDED'     => [
             qr{^inc/},
             qw( t/00sig.t
@@ -785,8 +758,8 @@ use File::Glob qw(:case);
     },
 
     'Math::BigRat' => {
-        'DISTRIBUTION' => 'PJACKLAM/Math-BigRat-0.2606.tar.gz',
-        'FILES'        => q[dist/Math-BigRat],
+        'DISTRIBUTION' => 'PJACKLAM/Math-BigRat-0.260801.tar.gz',
+        'FILES'        => q[cpan/Math-BigRat],
         'EXCLUDED'     => [
             qr{^inc/},
             qw( t/00sig.t
@@ -795,6 +768,10 @@ use File::Glob qw(:case);
                 t/03podcov.t
                 ),
         ],
+        'CUSTOMIZED'   => [
+            qw( lib/Math/BigRat.pm
+                ),
+        ],
     },
 
     'Math::Complex' => {
@@ -820,7 +797,7 @@ use File::Glob qw(:case);
     },
 
     'Module::CoreList' => {
-        'DISTRIBUTION' => 'BINGOS/Module-CoreList-5.20150720.tar.gz',
+        'DISTRIBUTION' => 'BINGOS/Module-CoreList-5.20151120.tar.gz',
         'FILES'        => q[dist/Module-CoreList],
     },
 
@@ -840,11 +817,12 @@ use File::Glob qw(:case);
     },
 
     'Module::Metadata' => {
-        'DISTRIBUTION' => 'ETHER/Module-Metadata-1.000027.tar.gz',
+        'DISTRIBUTION' => 'ETHER/Module-Metadata-1.000031-TRIAL.tar.gz',
         'FILES'        => q[cpan/Module-Metadata],
         'EXCLUDED'     => [
             qw(t/00-report-prereqs.t),
             qw(t/00-report-prereqs.dd),
+            qr{weaver.ini},
             qr{^xt},
         ],
     },
@@ -881,13 +859,16 @@ use File::Glob qw(:case);
     },
 
     'PathTools' => {
-        'DISTRIBUTION' => 'SMUELLER/PathTools-3.47.tar.gz',
+        'DISTRIBUTION' => 'RJBS/PathTools-3.60.tar.gz',
         'FILES'        => q[dist/PathTools],
-        'EXCLUDED'     => [qr{^t/lib/Test/}],
+        'EXCLUDED'     => [
+            qr{^t/lib/Test/},
+            qw( t/rel2abs_vs_symlink.t),
+        ],
     },
 
     'Perl::OSType' => {
-        'DISTRIBUTION' => 'DAGOLDEN/Perl-OSType-1.008.tar.gz',
+        'DISTRIBUTION' => 'DAGOLDEN/Perl-OSType-1.009.tar.gz',
         'FILES'        => q[cpan/Perl-OSType],
         'EXCLUDED'     => [qw(tidyall.ini), qr/^xt/, qr{^t/00-}],
     },
@@ -935,25 +916,15 @@ use File::Glob qw(:case);
         # XXX We can and should fix this, but clean up the DRY-failure in utils
         # first
         'EXCLUDED' => ['perldoc'],
+
+        # https://rt.cpan.org/Ticket/Display.html?id=106798
+        # https://rt.cpan.org/Ticket/Display.html?id=110368
+        'CUSTOMIZED'   => [ qw[ lib/Pod/Perldoc.pm ] ],
     },
 
     'Pod::Simple' => {
-        'DISTRIBUTION' => 'DWHEELER/Pod-Simple-3.30.tar.gz',
+        'DISTRIBUTION' => 'MARCGREEN/Pod-Simple-3.32.tar.gz',
         'FILES'        => q[cpan/Pod-Simple],
-        # https://rt.cpan.org/Public/Bug/Display.html?id=103439
-        # https://rt.cpan.org/Public/Bug/Display.html?id=105192
-        # https://rt.cpan.org/Public/Bug/Display.html?id=105511
-        'CUSTOMIZED'   => [
-            qw( lib/Pod/Simple/Search.pm
-                lib/Pod/Simple/BlackBox.pm
-                t/search10.t
-                t/search12.t
-                t/search20.t
-                t/search22.t
-                t/search26.t
-                t/search28.t
-            ),
-        ],
     },
 
     'Pod::Usage' => {
@@ -1018,11 +989,18 @@ use File::Glob qw(:case);
     'Socket' => {
         'DISTRIBUTION' => 'PEVANS/Socket-2.020.tar.gz',
         'FILES'        => q[cpan/Socket],
+
+        # https://rt.cpan.org/Ticket/Display.html?id=106797
+        # https://rt.cpan.org/Ticket/Display.html?id=107058
+        'CUSTOMIZED'   => [ qw[ Socket.pm Socket.xs ] ],
     },
 
     'Storable' => {
         'DISTRIBUTION' => 'AMS/Storable-2.51.tar.gz',
         'FILES'        => q[dist/Storable],
+        'EXCLUDED'     => [
+            qr{^t/compat/},
+        ],
     },
 
     'Sys::Syslog' => {
@@ -1043,7 +1021,7 @@ use File::Glob qw(:case);
     },
 
     'Term::ANSIColor' => {
-        'DISTRIBUTION' => 'RRA/Term-ANSIColor-4.03.tar.gz',
+        'DISTRIBUTION' => 'RRA/Term-ANSIColor-4.04.tar.gz',
         'FILES'        => q[cpan/Term-ANSIColor],
         'EXCLUDED'     => [
             qr{^examples/},
@@ -1073,7 +1051,7 @@ use File::Glob qw(:case);
 
     'Test' => {
         'DISTRIBUTION' => 'JESSE/Test-1.26.tar.gz',
-        'FILES'        => q[cpan/Test],
+        'FILES'        => q[dist/Test],
     },
 
     'Test::Harness' => {
@@ -1131,13 +1109,6 @@ use File::Glob qw(:case);
     'Text::ParseWords' => {
         'DISTRIBUTION' => 'CHORNY/Text-ParseWords-3.30.tar.gz',
         'FILES'        => q[cpan/Text-ParseWords],
-
-        # Waiting to be merged upstream:
-        # see https://github.com/chorny/Text-ParseWords/pull/6
-        'CUSTOMIZED'   => [
-            qw( t/ParseWords.t
-                ),
-        ],
     },
 
     'Text-Tabs+Wrap' => {
@@ -1159,7 +1130,7 @@ use File::Glob qw(:case);
     # correct for this (and Thread::Semaphore, threads, and threads::shared)
     # to be under dist/ rather than cpan/
     'Thread::Queue' => {
-        'DISTRIBUTION' => 'JDHEDDEN/Thread-Queue-3.05.tar.gz',
+        'DISTRIBUTION' => 'JDHEDDEN/Thread-Queue-3.07.tar.gz',
         'FILES'        => q[dist/Thread-Queue],
         'EXCLUDED'     => [
             qr{^examples/},
@@ -1217,7 +1188,7 @@ use File::Glob qw(:case);
     },
 
     'Time::HiRes' => {
-        'DISTRIBUTION' => 'ZEFRAM/Time-HiRes-1.9726.tar.gz',
+        'DISTRIBUTION' => 'RJBS/Time-HiRes-1.9728.tar.gz',
         'FILES'        => q[cpan/Time-HiRes],
     },
 
@@ -1230,7 +1201,7 @@ use File::Glob qw(:case);
     },
 
     'Time::Piece' => {
-        'DISTRIBUTION' => 'RJBS/Time-Piece-1.30.tar.gz',
+        'DISTRIBUTION' => 'ESAYM/Time-Piece-1.31.tar.gz',
         'FILES'        => q[cpan/Time-Piece],
     },
 
@@ -1249,7 +1220,7 @@ use File::Glob qw(:case);
     },
 
     'Unicode::Normalize' => {
-        'DISTRIBUTION' => 'SADAHIRO/Unicode-Normalize-1.19.tar.gz',
+        'DISTRIBUTION' => 'KHW/Unicode-Normalize-1.24.tar.gz',
         'FILES'        => q[cpan/Unicode-Normalize],
     },
 
@@ -1261,7 +1232,9 @@ use File::Glob qw(:case);
             'vutil/Makefile.PL',
             'vutil/ppport.h',
             'vutil/vxs.xs',
+            't/00impl-pp.t',
             't/survey_locales',
+            'lib/version/vpp.pm',
         ],
 
         # When adding the CPAN-distributed files for version.pm, it is necessary
@@ -1293,7 +1266,7 @@ use File::Glob qw(:case);
     },
 
     'Win32' => {
-        'DISTRIBUTION' => "JDB/Win32-0.51.tar.gz",
+        'DISTRIBUTION' => "JDB/Win32-0.52.tar.gz",
         'FILES'        => q[cpan/Win32],
     },
 
@@ -1346,6 +1319,8 @@ use File::Glob qw(:case);
 
     '_PERLLIB' => {
         'FILES'    => q[
+                ext/Amiga-ARexx/
+                ext/Amiga-Exec/
                 ext/B/
                 ext/Devel-Peek/
                 ext/DynaLoader/
