@@ -161,8 +161,11 @@ d_fpclassl='undef'
 d_fpgetround='undef'
 d_fpos64_t='undef'
 d_frexpl='undef'
+d_fs_data_s='undef'
 d_fseeko='undef'
 d_fsetpos='undef'
+d_fstatfs='undef'
+d_fstatvfs='undef'
 d_fsync='undef'
 d_ftello='undef'
 d_ftime='undef'
@@ -172,6 +175,7 @@ d_gdbmndbm_h_uses_prototypes='undef'
 d_getaddrinfo='undef'
 d_getcwd='undef'
 d_getespwnam='undef'
+d_getfsstat='undef'
 d_getgrent='undef'
 d_getgrent_r='undef'
 d_getgrgid_r='undef'
@@ -188,6 +192,8 @@ d_gethostprotos='undef'
 d_getitimer='undef'
 d_getlogin='undef'
 d_getlogin_r='undef'
+d_getmnt='undef'
+d_getmntent='undef'
 d_getnameinfo='undef'
 d_getnbyaddr='undef'
 d_getnbyname='undef'
@@ -229,6 +235,7 @@ d_gmtime64='undef'
 d_gmtime_r='undef'
 d_gnulibc='undef'
 d_grpasswd='undef'
+d_hasmntopt='undef'
 d_htonl='undef'
 d_hypot='undef'
 d_ilogb='undef'
@@ -454,7 +461,10 @@ d_sresgproto='undef'
 d_sresuproto='undef'
 d_stat='define'
 d_statblks='undef'
+d_statfs_f_flags='undef'
+d_statfs_s='undef'
 d_static_inline='undef'
+d_statvfs='undef'
 d_stdio_cnt_lval='undef'
 d_stdio_ptr_lval='undef'
 d_stdio_ptr_lval_nochange_cnt='undef'
@@ -513,6 +523,7 @@ d_unordered='undef'
 d_unsetenv='undef'
 d_usleep='undef'
 d_usleepproto='undef'
+d_ustat='undef'
 d_vendorarch='undef'
 d_vendorbin='undef'
 d_vendorlib='undef'
@@ -627,6 +638,7 @@ i_malloc='undef'
 i_mallocmalloc='undef'
 i_math='define'
 i_memory='undef'
+i_mntent='undef'
 i_ndbm='undef'
 i_netdb='undef'
 i_neterrno='undef'
@@ -657,6 +669,7 @@ i_sysioctl='undef'
 i_syslog='undef'
 i_sysmman='undef'
 i_sysmode='undef'
+i_sysmount='undef'
 i_sysndir='undef'
 i_sysparam='undef'
 i_syspoll='undef'
@@ -665,6 +678,8 @@ i_syssecrt='undef'
 i_sysselct='undef'
 i_syssockio='undef'
 i_sysstat='define'
+i_sysstatfs='undef'
+i_sysstatvfs='undef'
 i_systime='undef'
 i_systimek='undef'
 i_systimes='undef'
@@ -672,11 +687,13 @@ i_systypes='undef'
 i_sysuio='undef'
 i_sysun='undef'
 i_sysutsname='undef'
+i_sysvfs='undef'
 i_syswait='undef'
 i_termio='undef'
 i_termios='undef'
 i_time='define'
 i_unistd='undef'
+i_ustat='undef'
 i_utime='undef'
 i_values='undef'
 i_varargs='undef'
